#import "NSString+CocoaDevUsersAdditionsTest.h"
#import "NSString+CocoaDevUsersAdditions.h"

#ifndef UNIT_KIT
#define UKIntsEqual(expected, actual) [self assertInt:actual equals:expected]
#endif

@implementation NSString_CocoaDevUsersAdditionsTest

#ifndef UNIT_KIT
- (void)setUp
{
}

- (void)tearDown
{
}
#endif

- (void)testObjCTokens
{
   NSString* source =
        @"//  ObjCSource.h\n"
        @"\n"
        @"#import <Cocoa/Cocoa.h>\n"
        @"\n"
        @"\n"
        @"@interface ObjCSource : NSObject {\n"
        @"\n"    
        @"}\n"
        @"\n"
        @"-(void*)test:(id)param;\n"
        @"\n"
        @"@end\n";
    NSArray* tokens = [NSArray arrayWithObjects:
        @"ObjCSource", @"h",
        @"import", @"Cocoa", @"Cocoa", @"h",
        @"interface", @"ObjCSource", @":", @"NSObject",
        @"void", @"test:", @"id", @"param",
        @"end",
        nil];

    UKObjectsEqual(tokens, [source objCTokens]);
}

- (void)testWords
{
    NSString* text = @"7 November 2003 - added splitToSize:(unsigned)size, removeTabsAndReturns, newlineToCR methods";
    NSArray* words = [NSArray arrayWithObjects:
        @"November",  @"-", @"added", @"splitToSize", @"unsigned", @"size", @"removeTabsAndReturns", @"newlineToCR", @"methods", nil];
    
    UKObjectsEqual(words, [text words]);
}

- (void)testShellWords
{
    NSString* text = @"grep -v --   -apattern   file1\t\tfile2 \t file3";
    NSArray* words = [NSArray arrayWithObjects:
        @"grep", @"-v", @"--", @"-apattern", @"file", @"file", @"file", nil];

    UKObjectsEqual(words, [text words]);
}

- (void)testLineCount
{
    // empty string
    UKIntsEqual(0, [@"" lineCount]);
    
    // one line
    UKIntsEqual(1, [@"\n" lineCount]);
    UKIntsEqual(1, [@"First" lineCount]);    
    UKIntsEqual(1, [@"First\n" lineCount]);
    
    // multiply lines
    UKIntsEqual(2, [@"\nFirst\n" lineCount]);
    UKIntsEqual(2, [@"First\nSecond" lineCount]);
    UKIntsEqual(2, [@"First\nSecond\n" lineCount]);
    
    // empty lines between
    UKIntsEqual(3, [@"First\n\nSecond\n" lineCount]);
}

- (void)testFindRegex
{
    NSString* re = @"^.*[^[:blank:]][[:blank:]]+([[:digit:]]+)%[[:blank:]]+([[:digit:]]+(KB|MB)?)[[:blank:]]+([[:digit:]]+[[:punct:]][[:digit:]]+(KB|MB)/s)[[:blank:]]+(--:--|[[:digit:]]{2}:[[:digit:]]{2})[[:blank:]]*(ETA)?$";
    NSString* str = @"/Network/Users/tmcintos/starwarsWhosOnFirst.w   0%    0     0.0KB/s   --:-- ETA";
    NSArray*  arr = [NSArray arrayWithObjects: @"/Network/Users/tmcintos/starwarsWhosOnFirst.w   0%    0     0.0KB/s   --:-- ETA",
                                               @"0", @"0", [NSNull null], @"0.0KB/s", @"KB", @"--:--", @"ETA", nil];

    UKObjectsEqual(arr, [str findRegularExpression: re ignoreCase: YES]);

    NSString* excpname = nil;

    NS_DURING
        [str findRegularExpression: @"[" ignoreCase: YES];
    NS_HANDLER
        excpname = [[localException name] copy];
        NSLog(@"Caught exception: \"%@\"", [localException reason]);
    NS_ENDHANDLER

    UKObjectsEqual(NSInvalidArgumentException, excpname);
}

@end
