//
//  GDSProgressCell.m
//  Fugu
//
//  Created by Tim McIntosh on 2005.11.02.
//  Copyright 2005 __MyCompanyName__. All rights reserved.
//

#import "GDSProgressCell.h"

@implementation GDSProgressCell

static NSString* const ArrangedObjectsKeyPath = @"arrangedObjects";

- (void)setIndicators:(NSMutableArray*)a
{
    [mIndicators makeObjectsPerformSelector: @selector(removeFromSuperview)];
    [mIndicators release];
    mIndicators = [a retain];
}

- (id)init
{
    [super init];

    [self setIndicators: [NSMutableArray array]];

    return self;
}

- (void) dealloc
{
    [mController removeObserver: self forKeyPath: ArrangedObjectsKeyPath];

    [self setIndicators: nil];

    [super dealloc];
}

- (void)awakeFromNib
{
    [mController addObserver: self forKeyPath: ArrangedObjectsKeyPath options: 0 context: ArrangedObjectsKeyPath];
    [mTableColumn setDataCell: self];
}

- (NSString *)placeholderString
{
//    NSLog(@"%s: entered", __FUNCTION__);
    return @"";
}

- (void)setPlaceholderString:(NSString *)string
{
//    NSLog(@"%s: entered", __FUNCTION__);
}

- (void)arrangedObjectsChanged:(NSArray*)objects
{
    int count = [objects count];

//    NSLog(@"%s: number of rows = %d", __FUNCTION__, count);

    while ([mIndicators count] > count)
    {
        [[mIndicators lastObject] removeFromSuperview];
        [mIndicators removeLastObject];
    }

    while ([mIndicators count] < count)
    {
        NSProgressIndicator* p = [[NSProgressIndicator alloc] init];

        [p startAnimation: nil];

        [mIndicators addObject: p];

        [p release];
    }
}

- (void)observeValueForKeyPath:(NSString *)keyPath ofObject:(id)object change:(NSDictionary *)change context:(void *)context
{
    [self arrangedObjectsChanged: [mController arrangedObjects]];
}

- (void)drawInteriorWithFrame:(NSRect)cellFrame inView:(NSView *)controlView
{
    int row = [(NSTableView*)controlView rowAtPoint: cellFrame.origin];

//    NSLog(@"%s entered, row=%d, value=%@", __FUNCTION__, row, [self objectValue]);

    NSProgressIndicator* v = [mIndicators objectAtIndex: row];

    [v setFrame: [self titleRectForBounds: cellFrame]];

    if ([v superview] != controlView)
    {
        [controlView addSubview: v];
    }

    double value = [self objectValue] ? [[self objectValue] doubleValue] : 0;

    [v setDoubleValue: value];
    [v setIndeterminate: value < 0];
}

@end
