//
//  GDSPseudoTerminal.m
//  Fugu
//
//  Created by Tim McIntosh on 2005.03.18.
//  Copyright 2005 __MyCompanyName__. All rights reserved.
//

#import "GDSPseudoTerminal.h"

#include <errno.h>
#include <limits.h>
#include <termios.h>
#include <unistd.h>
#include <util.h>


@implementation GDSPseudoTerminal

- (id)init
{
    if ( self = [super init] )
    {
        int  masterFd, slaveFd;
        char ptyname[PATH_MAX];
        
        if ( openpty(&masterFd, &slaveFd, ptyname, NULL, NULL) == -1 )
        {
            [super release];
            
            [NSException raise: NSGenericException
                        format: @"openpty: %s", strerror(errno)];
        }

        mMaster = [[NSFileHandle alloc] initWithFileDescriptor: masterFd
                                                closeOnDealloc: YES];
        mSlave  = [[NSFileHandle alloc] initWithFileDescriptor: slaveFd
                                                closeOnDealloc: YES];
        mName   = [[NSString alloc] initWithUTF8String: ptyname];
    }

    return self;
}

- (void)dealloc
{
    [mMaster release];
    [mSlave release];
    [mName release];

    [super dealloc];
}

- (NSString*)description
{
    return [NSString stringWithFormat: @"<%@: %@>", NSStringFromClass([self class]), [self path]];
}

- (NSFileHandle*)masterFileHandle
{
    return mMaster;
}

- (NSFileHandle*)slaveFileHandle
{
    return mSlave;
}

- (NSString*)path
{
    return mName;
}

- (void)makeRaw
{
    struct termios attr;
    int fd;

    fd = [[self masterFileHandle] fileDescriptor];

    tcgetattr(fd, &attr);
    cfmakeraw(&attr);
    tcsetattr(fd, TCSANOW | TCSAFLUSH | TCSASOFT, &attr);
}

@end

@implementation GDSPseudoTerminal (TaskAdditions)

+ (GDSPseudoTerminal*)pseudoTerminalByLaunchingTask: (NSTask*)aTask
{
    GDSPseudoTerminal* pty               = [[GDSPseudoTerminal alloc] init];
    NSArray*           startPtyArguments = [[NSArray arrayWithObject: [aTask launchPath]] arrayByAddingObjectsFromArray: [aTask arguments]];

    NS_DURING
        [aTask setLaunchPath: [[NSBundle bundleForClass: [self class]] pathForAuxiliaryExecutable: @"start_pty"]];
        [aTask setArguments: startPtyArguments];
        [aTask setStandardInput: [pty slaveFileHandle]];

        [aTask launch];
    NS_HANDLER
        [pty release];
        [localException raise];
    NS_ENDHANDLER

    return [pty autorelease];
}

@end
