//
//  GDSTaskController.h
//  Fugu
//
//  Created by Tim McIntosh on 2005.10.05.
//  Copyright 2005 Green Dome Software. All rights reserved.
//

#import <Foundation/Foundation.h>

#import "GDSPseudoTerminal.h"

typedef enum
{
    GDSTaskActionSuccess,
    GDSTaskActionFailure,
    GDSTaskActionExited,
    GDSTaskActionInProgress
}
GDSTaskActionOutcome;


extern NSString* const GDSTaskActionResultLastReadKey;     // value:  NSString
extern NSString* const GDSTaskActionResultRawOutputKey;    // value:  NSMutableString
extern NSString* const GDSTaskActionResultErrorStringKey;  // value:  NSString

@interface GDSTaskActionMonitor : NSObject
{
    NSMutableDictionary*   mResults;
    GDSTaskActionOutcome   mOutcome;
}

+ (GDSTaskActionMonitor*)monitor;

- (void)setOutcome:(GDSTaskActionOutcome)outcome;
- (GDSTaskActionOutcome)outcome;

- (void)setResults:(NSMutableDictionary*)results;
- (NSMutableDictionary*)results;

@end

// XXX - rename GDSUnixCommandController?  Only for use with commands that return string data
@interface GDSTaskController : NSObject
{
    GDSPseudoTerminal*     mTerminal;
    NSTask*                mTask;
    GDSTaskActionMonitor*  mCurrentActionMonitor;
    id                     mDelegate;
}

- (GDSTaskController*)initWithLaunchPath:(NSString*)path arguments:(NSArray*)arguments;

- (NSTask*)task;

- (GDSTaskActionMonitor*)launch;

- (GDSTaskActionMonitor*)sendCommand:(NSString*)command;
- (void)sendResponse:(NSString*)response;

- (void)updateActionMonitor:(GDSTaskActionMonitor*)monitor;

- (id)delegate;
- (void)setDelegate:(id)delegate;

@end


@interface NSObject(GDSTaskControllerDelegateMethods)

- (void)taskController:(GDSTaskController*)controller updateActionMonitor:(GDSTaskActionMonitor*)monitor;

@end
