/*
 * Copyright (c) 2003 Regents of The University of Michigan.
 * All Rights Reserved.  See COPYRIGHT.
 */

#import "NSFileManager(mktemp).h"

#include <sys/types.h>
#include <sys/param.h>
#include <sys/stat.h>
#include <errno.h>
#include <unistd.h>

@implementation NSFileManager(mktemp)

- ( NSString * )processTempDirectoryPath
{
    static NSString* path = nil;
    
    if ( path == nil )
    {
        path = [[ NSTemporaryDirectory() stringByAppendingPathComponent: [ [ NSProcessInfo processInfo ] processName ] ] retain];
    }

#warning Clean up to use Foundation APIs

    if ( mkdir( [path cString], 0700 ) < 0)
    {
        if ( errno != EEXIST ) {
            char errmsg[256];

            bzero(errmsg, sizeof errmsg);
            strerror_r(errno, errmsg, sizeof errmsg - 1);

            [ NSException raise: NSGenericException
                         format: @"mkdir(%@,0700): %s", path, errmsg ];
	}
    }
    
    return path;
}

- ( char * )template: ( NSString * )template
{
    NSString      *path = [ [ self processTempDirectoryPath ] stringByAppendingPathComponent: template ];
    NSMutableData *data = [ [ path dataUsingEncoding: NSUTF8StringEncoding] mutableCopy ];
    char           null = '\0';

    [ data appendBytes: &null length: 1 ];

    return (char *) [ [ data autorelease ] mutableBytes ];
}

- ( NSString * )makeTemporaryDirectoryWithMode: ( mode_t )mode
{
    char      *template = [ self template: @"tmp.XXXXXX" ];
    NSString  *result;

    if ( mkdtemp( template ) == NULL ) {
        char errmsg[256];

        bzero(errmsg, sizeof errmsg);
        strerror_r(errno, errmsg, sizeof errmsg - 1);

        [ NSException raise: NSGenericException
                     format: @"mkdtmp(%s): %s", template, errmsg ];
    }

    result = [ NSString stringWithUTF8String: template ];

    return result;
}

- ( NSString * )makeTemporaryFileWithExtension: ( NSString *)ext;
{
    NSString  *result;
    char      *template;
    int       fd;

    if ( [ ext length ] > 0 )
    {
        template  = [ self template: [ @"tmp.XXXXXX" stringByAppendingPathExtension: ext ] ];
        fd = mkstemps( template, [ ext length ] + 1 );
    }
    else
    {
        template  = [ self template: @"tmp.XXXXXX" ];
        fd = mkstemp( template );
    }
    

    if ( fd < 0)
    {
        char errmsg[256];
        
        bzero(errmsg, sizeof errmsg);
        strerror_r(errno, errmsg, sizeof errmsg - 1);
        
        [ NSException raise: NSGenericException
                     format: @"mkstmps(%s): %s", template, errmsg ];
    }

    close( fd );

    result = [ NSString stringWithUTF8String: template ];
    
    return result;
}

@end
