//
//  NSImage+SystemIcons.m
//  Fugu
//
//  Created by Tim McIntosh on 2005.10.23.
//  Copyright 2005 __MyCompanyName__. All rights reserved.
//

#import "NSImage+SystemIcons.h"

@implementation NSImage(SystemIcons)

// XXX - move elsewhere?
static void _DisposeHandle(Handle h, const char* hstr)
{
    OSErr err;
    
    DisposeHandle(h);
    
    if ( (err = MemError()) != noErr )
    {
        NSLog(@"DisposeHandle(%s) returned error %d", hstr, err);
    }
}

#define DisposeHandle(h) _DisposeHandle(h, #h)

// Raises NSInavidArgumentException if unknown icon is requested
// Returns nil if failed to load icon data.

+ (NSImage*)imageForSystemIcon:(ResType)standardIconResource
{
    NSImage* theImage = nil;
    IconRef  theIconRef;
    OSErr    err;
    
    err = GetIconRef(kOnSystemDisk, kSystemIconsCreator, standardIconResource, &theIconRef);
    
    if ( err != noErr )
    {
        if ( err == noSuchIconErr )
        {
            char c1 = standardIconResource >> 24;
            char c2 = standardIconResource >> 16;
            char c3 = standardIconResource >> 8;
            char c4 = standardIconResource;
            
            [NSException raise: NSInvalidArgumentException
                        format: @"Unknown system icon: %c%c%c%c", c1, c2, c3, c4];
        }
    }
    else
    {
        IconFamilyHandle  iconFamily;
        
        err = IconRefToIconFamily(theIconRef, kSelectorAllAvailableData, &iconFamily);
        
        if ( err != noErr || iconFamily == NULL )
        {
            NSLog(@"IconRefToIconFamily failed: err=%d iconFamily=%p", err, iconFamily);
        }
        else
        {
            theImage = [[[self alloc] initWithData: [NSData dataWithBytes: *iconFamily
                                                                      length: GetHandleSize((Handle)iconFamily)]] autorelease];

            [theImage setScalesWhenResized: YES];

            DisposeHandle((Handle)iconFamily);
        }
        
        err = ReleaseIconRef(theIconRef);
        
        if ( err != noErr )
        {
            NSLog(@"ReleaseIconRef returned error %d", err);
        }
    }

    return theImage;
}

+ (NSImage*)imageForFinderIconNamed:(NSString*)name
{
    NSArray* coreServicesPaths = NSSearchPathForDirectoriesInDomains(NSCoreServiceDirectory, NSSystemDomainMask, NO);

    NSAssert([coreServicesPaths count] == 1, @"CoreServices path count != 1");

    NSString* finderPath = [[coreServicesPaths objectAtIndex: 0] stringByAppendingPathComponent: @"Finder.app"];

    // XXX returned path for CoreServices doesn't begin with /System!
    if (![finderPath hasPrefix: @"/System"])
    {
        finderPath = [@"/System" stringByAppendingPathComponent: finderPath];
    }

    NSBundle* finderBundle = [NSBundle bundleWithPath: finderPath];

    NSAssert1(finderBundle != nil, @"Finder bundle is nil, path=%@", finderPath);

    NSString* path = [finderBundle pathForImageResource: name];

    if (path == nil)
    {
        [NSException raise: NSInvalidArgumentException
                    format: @"Icon not found: %@", name];
    }
    
    NSImage* theImage = [[NSImage alloc] initWithContentsOfFile: path];

    [theImage setScalesWhenResized: YES];

    return [theImage autorelease];
}

@end
