//
//  NSString+SFTPAdditions.m
//  Fugu
//
//  Created by Tim McIntosh on 2005.10.08.
//  Copyright 2005 __MyCompanyName__. All rights reserved.
//

#import "NSString+SFTPAdditions.h"
#import "NSString+CocoaDevUsersAdditions.h"

// FIXME - merge into existing string additions category.

@implementation NSString (SFTPAdditions)

- (BOOL)containsAnyStringInArray:(NSArray*)strings
{
    NSEnumerator* e = [strings objectEnumerator];
    NSString* s;

    while ((s = [e nextObject]) != nil)
    {
        if ([self containsString: s])
            return YES;
    }

    return NO;
}

- (BOOL)containsSftpPasswordPrompt
{
    /* XXX someday we'll allow custom prompt checks */
    NSArray* prompts = [NSArray arrayWithObjects:
        @"Response:",                // SSH v1 challenge/response auth prompt
        @"password",
        @"Password:",
        @"passphrase",
        @"PASSCODE:",
        @"Password for ",
        @"Passcode for ",
        @"CryptoCard Challenge",
        nil];

    return [self containsAnyStringInArray: prompts];
}

- (BOOL)containsSftpError
{
    NSArray* errors = [NSArray arrayWithObjects:
        @"Permission denied",
        @"Couldn't ",
        @" not found.",
        @"Secure connection ",
        @"No address associated with",
        @"Connection refused",
        @"Request for subsystem",
        @"Cannot download",
        @"ssh_exchange_identification",
        @"Operation timed out"
        @"no address associated with",
        @"REMOTE HOST IDENTIFICATION HAS CHANGED",
        nil];
    
    return [self containsAnyStringInArray: errors];
}

- (BOOL)containsUnknownHostKeyPrompt
{
    NSArray* prompts = [NSArray arrayWithObjects:
        @"The authenticity of ",
        @"Host key not found ",
        nil];

    return [self containsAnyStringInArray: prompts];
}

- (BOOL)containsSftpPrompt
{
    return [self hasSuffix: @"sftp> "];    // XXX are there other prompts?
}

- (BOOL)containsConnectionClosed
{
    return [self containsString:@"Connection closed"];
}

NSString* const kSftpDirectoryFileType    = @"directory";
NSString* const kSftpFileFileType         = @"file";
NSString* const kSftpNamedPipeFileType    = @"named pipe";
NSString* const kSftpSocketFileType       = @"socket";
NSString* const kSftpBlockDevFileType     = @"block special";
NSString* const kSftpCharDevFileType      = @"char special";
NSString* const kSftpSymbolicLinkFileType = @"symbolic link";
NSString* const kSftpDoorFileType         = @"door";
NSString* const kSftpUnknownFileType      = @"unknown";

- (NSString*)sftpFileTypeStringFromPermissions
{
    switch ([self characterAtIndex: 0])
    {
        case 'd': return kSftpDirectoryFileType;
        case '-': return kSftpFileFileType;
        case 'p': return kSftpNamedPipeFileType;
        case 's': return kSftpSocketFileType;
        case 'b': return kSftpBlockDevFileType;
        case 'c': return kSftpCharDevFileType;
        case 'l': return kSftpSymbolicLinkFileType;
        case 'D': return kSftpDoorFileType;
        default: return kSftpUnknownFileType;
    }
}

@end
