/*
 * Copyright (c) 2003 Regents of The University of Michigan.
 * All Rights Reserved.  See COPYRIGHT.
 */

#import <Cocoa/Cocoa.h>

@class SFTPController;
@class GDSPseudoTerminal;

@protocol SFTPTServerInterface

- ( void )connectToServerWithParams: ( NSArray * )params
                     fromController: ( SFTPController * )controller;
- ( void )kill;

- ( int )atSftpPrompt;

- ( void )sendCommand: ( NSString * )cmd;
@end

@interface SFTPTServer : NSObject <SFTPTServerInterface> {
@private
    GDSPseudoTerminal   *pseudoTerminal;
    NSTask		*sftpTask;
    SFTPController	*controller;
    NSString		*_currentTransferName;
    NSString            *_sftpRemoteObjectList;

    int			threestrikes;
    BOOL                sethomedir;
    BOOL                pwsent;
    BOOL                validpw;
    BOOL		atprompt;
    BOOL                showrb;
    BOOL		was_uploading;
    BOOL                was_downloading;
    BOOL                was_changing;
    BOOL		was_removing;
    BOOL                was_renaming;
    BOOL                was_listing;
}

+ ( void )connectWithPorts: ( NSArray * )ports;
- ( id )init;
- ( NSString * )retrieveUnknownHostKey;
- ( NSMutableDictionary * )remoteObjectFromSFTPLine: ( const char * )line;
- ( BOOL )checkForPasswordPromptInBuffer: ( const char * )buf;
- ( BOOL )hasDirectoryListingFormInBuffer: ( const char * )buf;
- ( void )collectListing;

@end
