/*
 * Copyright (c) 2003 Regents of The University of Michigan.
 * All Rights Reserved.  See COPYRIGHT.
 */

// FIXME - make this into app controller, sftpcontroller into document controller - move Help menu stuff here. -TKM

#import "SFTPAppController.h"

#import "SFTPController.h"
#import "SCPController.h"
#import "SSHTunnel.h"
#import "SFTPPrefs.h"

#import "SFTPClientTaskController.h"

#import "MainMenuTags.h"

#import "UMVersionCheck.h"

#import "NSFileManager(mktemp).h"

#include <netinet/in.h>
//#include <netdb.h>
#include <arpa/inet.h>


#ifndef GNUSTEP
@interface NSDictionary (NSNetService)
+ (NSDictionary*)serviceWithHostName:(NSString*)name port:(uint16_t)port;
- (NSString*)name;
- (NSNumber*)port;
@end

@implementation NSDictionary (NSNetService)

+ (NSDictionary*)serviceWithHostName:(NSString*)name port:(uint16_t)port
{
    return [ NSDictionary dictionaryWithObjectsAndKeys:
        [ name copy ], @"name",
        [ NSNumber numberWithUnsignedShort: port ], @"port",
        nil ];
}

- (NSString*)name  // XXX conflicts with file name category
{
    return [ self objectForKey: @"name" ];
}

- (NSNumber*)port
{
    return [ self objectForKey: @"port" ];
}

@end
#endif

@implementation SFTPAppController

- ( void )awakeFromNib
{
#ifndef GNUSTEP
    services = [[ NSMutableDictionary alloc ] init];
#endif

    [ self newDocument: nil ];
}

- (void) dealloc
{
#ifndef GNUSTEP
    [ sshServiceBrowser release ];
    [ services release ];
#endif
    
    [ prefs release ];
    [ scp release ];

    [super dealloc];
}

#pragma mark -- Attribute Accessors --

- ( SCPController * )scp
{
    if ( scp == nil )
    {
        scp = [[ SCPController alloc ] init ];
        [ NSBundle loadNibNamed: @"SCP" owner: scp ];
    }
    
    return scp;
}

#ifndef GNUSTEP
- ( NSArray * )rendezvousServers
{
    return [ services allValues ];
}

- ( void )scanForSSHServers
{
    /* find rendezvous-enabled ssh servers, populate list with them */
    if ( sshServiceBrowser == nil )
    {
	sshServiceBrowser = [[ NSNetServiceBrowser alloc ] init ];
    }
    
    [ sshServiceBrowser setDelegate: self ];
    [ sshServiceBrowser searchForServicesOfType: @"_ssh._tcp" inDomain: @"" ];
}
#endif

#pragma mark -- NSApplication Delegate Methods --

- ( void )applicationDidFinishLaunching: ( NSNotification * )aNotification
{
    [ NSApp setServicesProvider: self ];
}

- ( BOOL )application: ( NSApplication * )theApplication
             openFile: ( NSString * )aFileName
{
    if ( scp == nil ) {
        scp = [[ SCPController alloc ] init ];
        [ NSBundle loadNibNamed: @"SCP" owner: scp ];
    }
    
    [ scp getSecureCopyWindowForFile: aFileName scpType: 0 copyToPath: @""
                            fromHost: @"" userName: @"" delegate: self ];
    
    return( YES );
}

- (NSApplicationTerminateReply)applicationShouldTerminate:(NSApplication *)sender
{
    NSEnumerator* e = [[ sender windows ] objectEnumerator ];
    NSWindow* w;

    while ( ( w = [ e nextObject ] ) != nil )
    {
        id d = [[ w delegate ] retain ];

        if ( [ w isMemberOfClass: [ NSWindow class ]] && [ d isKindOfClass: [ SFTPController class ]] )
        {
            if ([ (SFTPClientTaskController*)[ d client ] isConnected ])
            {
                [ d disconnect: nil ];

                if ([ (SFTPClientTaskController*)[ d client ] isConnected ])
                {
                    return NSTerminateCancel;
                }
            }
        }

        [ d release ];
    }

    return NSTerminateNow;
}

- ( void )applicationWillTerminate: ( NSNotification * )aNotification
{
    NSString*  tmpDir = [[ NSFileManager defaultManager ] processTempDirectoryPath ];  // Note: current implementation creates directory each time this is called.
    
    [[ NSFileManager defaultManager ] removeFileAtPath: tmpDir handler: nil ];
    
    if ( [[ NSFileManager defaultManager ] fileExistsAtPath: tmpDir ] )
    {
        NSLog( @"Couldn't remove %@!", tmpDir );
    }
    
#warning XXX - fix this - tkm
#if 0
    int			i;
    NSArray		*array;
    NSMutableArray	*columnArray = nil;
    
    /* save column settings */
    array = [ localBrowser tableColumns ];
    columnArray = [[ NSMutableArray alloc ] init ];
    
    for ( i = 1; i < [ array count ]; i++ ) {
        NSString	*identifier = [[ array objectAtIndex: i ] identifier ];
        float		width = [[ array objectAtIndex: i ] width ];
        
        [ columnArray addObject: [ NSDictionary dictionaryWithObjectsAndKeys:
            identifier, @"identifier",
            [ NSNumber numberWithFloat: width ], @"width", nil ]];
    }
    
    [[ NSUserDefaults standardUserDefaults ] setObject: columnArray
                                                forKey: @"VisibleColumns" ];
    [ columnArray release ];
#endif
}

#pragma mark -- Services --

/* provide scp service */
- ( void )secureCopyFile: ( NSPasteboard * )pboard
                userData: ( NSString * )userData
                   error: ( NSString ** )error
{
    NSArray		*filenames;
    NSArray		*types = [ pboard types ];
    
    if ( ![ types containsObject: NSFilenamesPboardType ] ) {
        *error = @"Pasteboard doesn't contain any filenames";
        return;
    }
    filenames = [ pboard propertyListForType:
        [ pboard availableTypeFromArray:
            [ NSArray arrayWithObject: NSFilenamesPboardType ]]];
    if ( filenames == nil ) {
        *error = @"Couldn't extract filename from pasteboard.";
        return;
    }
    
    if ( scp == nil ) {
        scp = [[ SCPController alloc ] init ];
        [ NSBundle loadNibNamed: @"SCP" owner: scp ];
    }
    
    [ scp getSecureCopyWindowForFile: [ filenames objectAtIndex: 0 ] 
                             scpType: 0 copyToPath: @"" fromHost: @"" userName: @"" delegate: self ];
}

/* provide ssh tunnel service */
- ( void )createSSHTunnel: ( NSPasteboard * )pboard
                 userData: ( NSString * )userData
                    error: ( NSString ** )error
{
    SSHTunnel		*ssht = [[ SSHTunnel alloc ] init ];
    
    [ NSBundle loadNibNamed: @"SSHTunnel" owner: ssht ];
    [ ssht displayWindow ];
    [ ssht autorelease ];
}

#pragma mark -- NSMenuValidation Protocol --

#if 0
- ( BOOL )validateMenuItem: ( NSMenuItem * )anItem
{
    NSLog(@"delegate validate: %@", anItem);

    return YES;
}
#endif

#pragma mark -- IBActions --

- (IBAction)newDocument:(id)sender
{
    [ [ SFTPController alloc ] init ];
}

- ( IBAction )emailAuthor: ( id )sender
{
    [[ NSWorkspace sharedWorkspace ] openURL: [ NSURL URLWithString:
                                        @"mailto:tmcintos@avalen.net" ]];
}

- ( IBAction )visitHomePage:( id )sender
{
    [[ NSWorkspace sharedWorkspace ] openURL: [ NSURL URLWithString:
                                        @"http://www.avalon.net/~tmcintos/software/Fugu/" ]];
}

- ( IBAction )showPrefs: ( id )sender
{
    if ( prefs == nil )
    {
        prefs = [[ SFTPPrefs alloc ] init ];
    }
    [ prefs showPreferencePanel ];
}

- ( IBAction )showItemFromHelpMenu: ( id )sender
{
    NSString		*path = nil;
    
    if ( [ sender isKindOfClass: [ NSMenuItem class ]] ) {
        switch ( [sender tag] )
        {
            case HELP_MENU_ITEM_README:
                path = [ NSBundle pathForResource: @"Fugu README" ofType: @"rtfd"
                                      inDirectory: [[ NSBundle mainBundle ] bundlePath ]];
                break;
                
            case HELP_MENU_ITEM_KEYS:
                path = [[ NSBundle mainBundle ] pathForResource: @"keys"
                                                         ofType: @"rtf" ];
                break;
                
            case HELP_MENU_ITEM_COPYRIGHT:
                path = [[ NSBundle mainBundle ] pathForResource: @"COPYRIGHT"
                                                         ofType: @"txt" ];
                break;
                
            default:
                [NSException raise: NSInternalInconsistencyException
                            format: @"Invalid Help menu item %d (%@)", [sender tag], [sender title]];
        }
        
        [ [NSWorkspace sharedWorkspace] openFile: path ];
    }
}

- ( IBAction )checkForUpdates: ( id )sender
{
    UMVersionCheck      *check = [[ UMVersionCheck alloc ] init ];
    
    [ check checkForUpdates ];
    [ check release ];
}

- ( IBAction )newSSHTunnel: ( id )sender
{
    SSHTunnel		*t = [[ SSHTunnel alloc ] init ];
    
    [ NSBundle loadNibNamed: @"SSHTunnel" owner: t ];
    [ t displayWindow ];
    [ t autorelease ];
}

- ( IBAction )secureCopy: ( id )sender
{
    [ [ self scp ] getSecureCopyWindowForFile: @""
                                      scpType: 0
                                   copyToPath: @""
                                     fromHost: @""
                                     userName: @"" 
                                     delegate: self ];
}

#pragma mark -- NSNetService delegate methods --
#ifndef GNUSTEP

- ( void )netService: ( NSNetService * )netService didNotResolve: ( NSDictionary * )errorDict
{
    // XXX what to do here... -TKM
    NSLog(@"Did not resolve address: %@", errorDict);

    [ netService release ];  // was retained below
}

- ( void )netServiceDidResolveAddress: ( NSNetService * )netService
{
    NSEnumerator*  e = [[ netService addresses ] objectEnumerator ];
    NSData*        address;
    
    while ( (address = [ e nextObject ]) != nil )
    {
        struct sockaddr* sa = ( struct sockaddr * )[ address bytes ];
        
        if (sa->sa_family != AF_INET )
        {
            NSLog( @"%s: %d: unknown address family.\n", __func__, sa->sa_family );
        }
        else
        {
            // Stop when internet address is found.
            break;
        }
    }

    if ( address == nil )
    {
        NSLog(@"No internet address for NetService.");
    }
    else
    {
        struct sockaddr_in* ip = ( struct sockaddr_in * )[ address bytes ];

        NSString* serviceAddress       = [ netService hostName ];
        uint16_t  port                 = ntohs( ip->sin_port );

        // XXX - should include port - TKM
        [ self willChangeValueForKey: @"rendezvousServers" ];
        [ services setObject: [ NSDictionary serviceWithHostName: serviceAddress port: port ]
                      forKey: serviceAddress ];
        [ self didChangeValueForKey: @"rendezvousServers" ];
    }
    
    [ netService release ];  // was retained below
}

#pragma mark -- NSNetServiceBrowser delegate methods --

- ( void )netServiceBrowser: ( NSNetServiceBrowser * )aNetServiceBrowser
             didFindService: ( NSNetService * )netService
                 moreComing: ( BOOL )moreComing
{
    [ netService retain ];

    [ netService setDelegate: self ];
    [ netService resolveWithTimeout: 30.0 ];
}

- ( void )netServiceBrowser: ( NSNetServiceBrowser * )aNetServiceBrowser
           didRemoveService: ( NSNetService * )netService
                 moreComing: ( BOOL )moreComing
{
    NSString* name = [ netService hostName ];

    if ( name != nil )
    {
        [ services removeObjectForKey: name ];
    }

    if ( ! moreComing )
    {
        [ self willChangeValueForKey: @"rendezvousServers" ];
        [ self didChangeValueForKey: @"rendezvousServers" ];
    }
}
#endif

@end
