//
//  SFTPClientTaskController.h
//  Fugu
//
//  Created by Tim McIntosh on 2005.10.06.
//  Copyright 2005 Green Dome Software. All rights reserved.
//

#import<Foundation/Foundation.h>

@class GDSTaskController;
@class SFTPCommand;

typedef enum
{
    SFTPClientTaskStateNotConnected,
    SFTPClientTaskStateConnecting,
    SFTPClientTaskStatePassword,
    SFTPClientTaskStateReady,
    SFTPClientTaskStateBusy
}
SFTPClientTaskState;

@interface SFTPClientTaskController : NSObject
{
@private
    id                     mActionMonitor;
    double                mSshVersion;
    NSString*              mUser;
    NSString*              mServer;
    NSNumber*              mPort;
    GDSTaskController*     mTaskController;
    SFTPClientTaskState    mState;
    NSString*              mError;
    NSString*              mPath;
    NSArray*               mFiles;
    SFTPCommand*           mCurrentCommand;
    NSMutableArray*        mCommandQueue;
#ifdef SSH_COM_SUPPORT
    NSArray*               mPartialResponse;
#endif
    BOOL                   mNeedsRefresh;
    id                     mDelegate;
}

+ (double)sshVersion;
+ (BOOL)hasSupportedSftpVersion;

- (SFTPClientTaskController*)initWithServer:(NSString*)server
                                       port:(NSNumber*)port
                                       user:(NSString*)user
                                    options:(NSArray*)options;

- (double)sshVersion;

- (NSString*)user;
- (NSString*)server;
- (NSNumber*)port;

- (NSString*)sessionName;

- (id)delegate;
- (void)setDelegate:(id)delegate;

- (void)connect;
- (void)disconnect;
- (BOOL)isConnected;

- (void)sendPassword:(NSString*)password;
- (void)sendUnknownHostKeyResponse:(BOOL)continueConnecting;

- (SFTPClientTaskState)state;
- (BOOL)isBusy;

- (void)queueCommand:(SFTPCommand*)command;
- (NSArray*)commandQueue;
- (SFTPCommand*)currentCommand;
- (void)flushQueue;

- (NSString*)error;
- (void)resume;

- (NSString*)path;
- (void)setPath:(NSString*)path;
- (void)changeToParentPath;

- (NSArray*)files;

- (BOOL)needsRefresh;
- (void)setNeedsRefresh:(BOOL)needsRefresh;
- (void)refresh;

@end

@interface NSObject(SFTPClientTaskControllerDelegateMethods)
- (void)sftpClient:(SFTPClientTaskController*)client readRawData:(NSString*)partialRead;
- (void)sftpClient:(SFTPClientTaskController*)client atUnknownHostKeyPrompt:(NSString*)prompt;
- (void)sftpClient:(SFTPClientTaskController*)client atPasswordPrompt:(NSString*)prompt;
- (void)sftpClientConnectionFailed:(SFTPClientTaskController*)client;
- (void)sftpClientConnected:(SFTPClientTaskController*)client;
- (void)sftpClientDisconnected:(SFTPClientTaskController*)client;
- (void)sftpClient:(SFTPClientTaskController*)client pausedDueToError:(NSString*)error;
@end

@interface SFTPClientTaskController(PackageMethods)
+ (NSString*)sftpLaunchPath;
+ (double)sshVersion;
- (void)setError:(NSString*)error;
- (void)setFiles:(NSArray*)files;
- (NSString*)absolutePathFromString:(NSString*)path;
@end

@interface NSDictionary(FileListMethods)

- (NSString*)name;
//- (NSDate*)date;  XXX
//- (NSNumber*)size; XXX
- (NSString*)date;
- (NSString*)size;
- (NSString*)type;
- (NSString*)perm;
- (NSString*)owner;
- (NSString*)group;

@end
