/*
 * Copyright (c) 2003 Regents of The University of Michigan.
 * All Rights Reserved.  See COPYRIGHT.
 */

#import <AppKit/AppKit.h>

#import "SFTPTableView.h"
#import "SFTPImagePreviewPanel.h"
#import "SFTPImageView.h"
#import "UKKQueue/UKKQueue.h"

@class SFTPClientTaskController;

@interface 	SFTPController : NSWindowController
{
#ifndef GNUSTEP
    IBOutlet NSObjectController*       mSessionController;
    IBOutlet NSArrayController*        mTasksController;
    IBOutlet NSArrayController*        mEditedDocumentsController;
#endif

    IBOutlet NSPanel                   *editedDocumentsPanel;
    IBOutlet NSPanel                   *activityPanel;

    /* console panel outlets */
    IBOutlet NSDrawer			*logDrawer;
    IBOutlet NSTextView		*logField;
    IBOutlet NSButton			*commandButton;
    IBOutlet NSTextField 		*manualComField;
    
    /* new dir panel outlets */
    IBOutlet NSPanel			*newDirPanel;
    IBOutlet NSTextField		*newDirNameField;
    IBOutlet NSButton			*newDirCreateButton;
    IBOutlet NSButton			*localDirCreateButton;
    IBOutlet NSButton			*remoteDirCreateButton;
    
    /* go to dir panel outlets */
    IBOutlet NSPanel			*gotoDirPanel;
    IBOutlet NSComboBox		*gotoDirNameField;
    IBOutlet NSButton			*gotoButton;
    IBOutlet NSButton			*localGotoButton;
    IBOutlet NSButton			*remoteGotoButton;
    
    /* upload progress panel outlets */
    IBOutlet NSPanel			*uploadProgPanel;
    IBOutlet NSTextField		*uploadProgName;
    IBOutlet NSTextField		*uploadProgItemsLeft;
    IBOutlet NSTextField		*uploadTimeField;
    IBOutlet NSProgressIndicator	*uploadProgBar;
    IBOutlet NSTextField		*uploadProgInfo;
    
    /* download progress panel outlets */
    IBOutlet NSPanel			*downloadSheet;
    IBOutlet NSTextField		*downloadTextField;
    IBOutlet NSTextField		*downloadTimeField;
    IBOutlet NSProgressIndicator	*downloadProgBar;
    IBOutlet NSTextField		*downloadProgInfo;
    IBOutlet NSTextField		*downloadProgPercentDone;
    
    /* preview panel outlets */
    IBOutlet SFTPImagePreviewPanel	*imagePreviewPanel;
    IBOutlet NSBox			*imagePreviewBox;
    IBOutlet NSView			*imagePreviewView;
    IBOutlet SFTPImageView		*imagePreview;
    IBOutlet NSTextField		*imagePreviewTextField;
    
    /* info panel outlets */
    IBOutlet NSPanel			*infoPanel;
    IBOutlet NSTabView			*infoTabView;
        // local info fields
    IBOutlet NSImageView		*largeIcon;
    IBOutlet NSTextField		*modDateField;
    IBOutlet NSTextField	 	*groupField;
    IBOutlet NSTextField 		*ownerField;
    IBOutlet NSTextField 		*sizeField;
    IBOutlet NSTextField 		*typeField;
    IBOutlet NSTextField 		*permField;
    IBOutlet NSTextField 		*infoPathField;
    IBOutlet NSTextField		*whereField;
    IBOutlet NSButton 			*loReadSwitch;
    IBOutlet NSButton 			*loWriteSwitch;
    IBOutlet NSButton 			*loExecSwitch;
    IBOutlet NSButton 			*lgReadSwitch;
    IBOutlet NSButton 			*lgWriteSwitch;
    IBOutlet NSButton 			*lgExecSwitch;
    IBOutlet NSButton 			*laReadSwitch;
    IBOutlet NSButton 			*laWriteSwitch;
    IBOutlet NSButton 			*laExecSwitch;
        // remote info fields
    IBOutlet NSImageView		*rIcon;
    IBOutlet NSTextField		*rModDateField;
    IBOutlet NSTextField	 	*rGroupField;
    IBOutlet NSTextField 		*rOwnerField;
    IBOutlet NSTextField 		*rSizeField;
    IBOutlet NSTextField 		*rTypeField;
    IBOutlet NSTextField 		*rPermField;
    IBOutlet NSTextField 		*rInfoPathField;
    IBOutlet NSTextField		*rWhereField;
    IBOutlet NSButton			*roReadSwitch;
    IBOutlet NSButton			*roWriteSwitch;
    IBOutlet NSButton			*roExecSwitch;
    IBOutlet NSButton			*rgReadSwitch;
    IBOutlet NSButton			*rgWriteSwitch;
    IBOutlet NSButton			*rgExecSwitch;
    IBOutlet NSButton			*raReadSwitch;
    IBOutlet NSButton			*raWriteSwitch;
    IBOutlet NSButton			*raExecSwitch;
    
    /* main ui elements */
    IBOutlet NSBox			*localBox;
    IBOutlet NSBox			*remoteBox;
    
    /* local and remote views */
    IBOutlet NSView			*localView;
    IBOutlet NSView			*remoteView;
    IBOutlet NSPopUpButton		*rPathPopUp;
    IBOutlet NSPopUpButton		*lPathPopUp;
    IBOutlet SFTPTableView 		*localBrowser;
    IBOutlet NSMenu			*localTableMenu;
    IBOutlet SFTPTableView	 	*remoteBrowser;
    IBOutlet NSMenu			*remoteTableMenu;
    
    IBOutlet id hostKeyUnauth;
    IBOutlet id unauthHostInfo;
    IBOutlet NSButton			*addToFavButton;
    IBOutlet id verifyConnectSheet;
    
    /* connecting view outlets */
    IBOutlet NSView			*connectingView;
    IBOutlet NSProgressIndicator	*connectingProgress;
    IBOutlet NSTextField		*connectingToField;
    
    /* password prompt outlets */
    IBOutlet NSTextField 		*passErrorField;
    IBOutlet NSView	 		*passView;
    IBOutlet NSTextField		*passWord;
    IBOutlet NSTextField		*passHeader;
    IBOutlet NSProgressIndicator	*authProgBar;
    IBOutlet NSButton			*passAuthButton;
    IBOutlet NSButton			*passCancelButton;
    IBOutlet NSButton			*addToKeychainSwitch;
    
    /* login view outlets */
    IBOutlet NSView			*loginView;
    IBOutlet NSPopUpButton 		*popUpFavs;
    IBOutlet id remoteHost;
    IBOutlet id userName;
    IBOutlet NSTextField 		*portField;
    IBOutlet NSTextField		*loginDirField;
    IBOutlet NSButton			*connectButton;
    IBOutlet NSPopUpButton		*rendezvousPopUp;
    IBOutlet NSBox			*favsConnectBox;
    
    /* advanced connection outlets */
    IBOutlet NSBox			*advConnectionBox;
    IBOutlet NSTextField		*advAdditionalOptionsField;
    IBOutlet NSButton			*advForceSSH1Switch;
    IBOutlet NSButton			*advEnableCompressionSwitch;

    /* File menu items */
    IBOutlet NSMenuItem			*localMenuOpenWithMenuItem;
    IBOutlet NSMenuItem			*remoteMenuOpenWithMenuItem;
    
@private

    NSDictionary*               mToolbarItems;

    SFTPClientTaskController*   mClient;

    float		        dltime, ultime;

    NSTimer                     *timer;
    NSMutableString             *typeAheadString;

    /*
     * used to tell if the password in the keychain was wrong.
     * if so, don't keep turning to the keychain for the password.
     * prompt the user as usual, instead.
     */
    BOOL			firstPasswordPrompt;
    BOOL			gotPasswordFromKeychain;

    int				dotflag;

    NSMenu			*openWithMenu;

    NSMutableDictionary 	*remoteFileInfo;
    NSString			*remoteHome;

    NSMutableArray		*remoteDirContents;
    NSMutableArray		*dotlessRDir;
    NSArray			*rSwitchArray;
    NSString			*remoteDirPath;
    NSMutableArray		*localDirContents;
    NSMutableArray		*dotlessLDir;
    NSString			*localDirPath;
    
    /* documents being edited from the server */
    NSMutableArray		*editedDocuments;
    unsigned int                lastEditedDocumentCount;
    
    /* images being previewed from the server */
    NSString			*previewedImage;
    NSMutableArray		*cachedPreviews;
    
    /* menus for history */
    NSMenu			*localHistoryMenu;
    NSMenu			*remoteHistoryMenu;

    /* images for display in browsers */
    NSImage			*dirImage;
    NSImage			*fileImage;
    NSImage			*linkImage;
    
    /* spring-loaded root path */
    NSString                    *_springLoadedRootPath;
    
    /* kernel event queue */
    UKKQueue                    *kqueue;
    
    /* drag destination for file promises from remote browser */
    NSURL                       *promisedFilesDestinationURL;
}

- ( SFTPClientTaskController* )client;

- ( NSDictionary* )localFileAtIndex: ( int )anIndex;
- ( NSDictionary* )remoteFileAtIndex: ( int )anIndex;

- ( NSDictionary* )localSelectedFile;
- ( NSDictionary* )remoteSelectedFile;

- ( void )showUploadProgress;
- ( void )updateUploadProgress;

- ( void )uploadDirectory: ( NSString * )localPath toDirectory: ( NSString * )remotePath;
- ( void )updateUploadProgressBarWithValue: ( double )value
	    amountTransfered: ( NSString * )amount
	    transferRate: ( NSString * )rate
	    ETA: ( NSString * )eta;

- ( IBAction )showEditedDocumentsViewer: ( id )sender;
- ( IBAction )showActivityViewer: ( id )sender;

- ( IBAction )sendPassword: ( id )sender;
- ( void )addPasswordToKeychain;
- ( BOOL )retrievePasswordFromKeychain;
- ( BOOL )firstPasswordPrompt;
- ( void )setFirstPasswordPrompt: ( BOOL )first;
- ( void )setGotPasswordFromKeychain: ( BOOL )rp;
- ( BOOL )gotPasswordFromKeychain;
- ( IBAction )cancelConnection: ( id )sender;
- ( IBAction )continueConnecting: ( id )sender;
- ( IBAction )toggleDots: ( id )sender;

- ( IBAction )toggleAdvConnectionView: ( id )sender;

- ( void )showConnectingInterface: ( id )sender;

- ( IBAction )toggleGoToButtons: ( id )sender;
- ( IBAction )getGotoDirPanel: ( id )sender;
- ( IBAction )gotoLocalDirectory: ( id )sender;
- ( IBAction )gotoRemoteDirectory: ( id )sender;
- ( IBAction )dismissGotoDirPanel: ( id )sender;

- ( IBAction )selectFromFavorites: ( id )sender;
- ( IBAction )addToFavorites: ( id )sender;

- ( IBAction )sftpConnect: ( id )sender;

- ( void )showDownloadProgressWithMessage: ( NSString * )msg;
- ( void )updateDownloadProgressBarWithValue: ( double )value
	    amountTransfered: ( NSString * )amount
	    transferRate: ( NSString * )rate
	    ETA: ( NSString * )eta;
- ( void )finishedDownload: ( NSString * )remotePath;

- ( NSMenu * )openWithMenu;

/* interaction related methods */
- ( void )uploadFiles: ( NSArray * )lfiles toDirectory: ( NSString * )rpath;
- ( void )downloadFiles: ( NSArray * )rpaths toDirectory: ( NSString * )lpath;
- ( IBAction )createNewDirectory: ( id )sender;
- ( IBAction )toggleDirCreationButtons: ( id )sender;
- ( IBAction )dismissNewDirPanel: ( id )sender;

/* spring-loaded folders */
- ( void )performSpringLoadedActionInTable: ( NSTableView * )table;
- ( void )setSpringLoadedRootPathInTable: ( NSTableView * )table;
- ( NSString * )springLoadedRootPath;

/* methods related to deleting items */
- ( IBAction )delete: ( id )sender;
- ( IBAction )deleteLocalFile: ( id )sender;
- ( IBAction )deleteRemoteFile: ( id )sender;
- ( void )deleteLocalFiles: ( NSArray* )files;
- ( void )deleteRemoteFiles: ( NSArray* )files;

- ( IBAction )focusOnLocalPane: ( id )sender;
- ( IBAction )focusOnRemotePane: ( id )sender;

- ( IBAction )localBrowserSingleClick: ( id )browser;
- ( IBAction )localBrowserDoubleClick: ( id )browser;
- ( IBAction )uploadButtonClick: ( id )sender;
- ( IBAction )remoteBrowserSingleClick: ( id )browser;
- ( IBAction )remoteBrowserDoubleClick: ( id )browser;
- ( IBAction )downloadButtonClick: ( id )sender;
- ( IBAction )cdRemoteHome: ( id )sender;
- ( IBAction )cdLocalHome: ( id )sender;
- ( IBAction )remoteCdDotDot: ( id )sender;
- ( IBAction )localCdDotDot: ( id )sender;
- ( IBAction )cdFromLPathPopUp: ( id )sender;
- ( IBAction )cdFromRPathPopUp: ( id )sender;
- ( IBAction )refreshBrowsers: ( id )sender;

- ( void )changeDirectory: ( id )sender;
- ( void )dotdot: ( id )sender;

- ( IBAction )changeROwner: ( id )sender;
- ( IBAction )changeRGroup: ( id )sender;
- ( IBAction )changeRemoteMode: ( id )sender;

- ( IBAction )changeLOwnerAndGroup: ( id )sender;
- ( IBAction )changeLocalMode: ( id )sender;

- ( IBAction )renameLocalItem: ( id )sender;
- ( IBAction )renameRemoteItem: ( id )sender;

#ifdef XXX // remove
- ( void )setBusyStatusWithMessage: ( NSString * )message;
#endif
- ( void )finishedCommand;
- ( IBAction )sendManualCommand: ( id )sender;

- ( void )clearLog;
- ( void )addToLog: ( NSString * )text;
- ( void )updateHostList;
- ( void )reloadDefaults;

- ( IBAction )disconnect: ( id )sender;
- (void)sftpClientDisconnected:(SFTPClientTaskController*)client;
- ( id )init;
//- ( void )mouseDown: ( NSEvent * )theEvent;

- ( void )localBrowserReloadForPath: ( NSString * )path;

- ( IBAction )toggleGroupColumn: ( id )sender;
- ( IBAction )toggleModDateColumn: ( id )sender;
- ( IBAction )toggleModeColumn: ( id )sender;
- ( IBAction )toggleOwnerColumn: ( id )sender;
- ( IBAction )toggleSizeColumn: ( id )sender;

- ( IBAction )toggleConsole: ( id )sender;

- ( void )setRemotePathPopUp: ( NSString * )pwd;
- ( void )loadRemoteBrowserWithItems: ( NSArray * )items;

- ( IBAction )getInfo: ( id )sender;

- ( void )remoteCheckSetup: ( NSString * )permissions;
- ( void )localCheckSetup: ( NSString * )octalmode;
- ( void )awakeFromNib;
- ( void )dealloc;

- ( int )matchingIndexForString: ( NSString * )string inTable: ( SFTPTableView * )table;
                
- ( IBAction )selectRendezvousServer: ( id )sender;

- ( IBAction )previewItem: ( id )sender;
- ( IBAction )previewLocalItem: ( id )sender;
- ( IBAction )previewRemoteItem: ( id )sender;
- ( void )setPreviewedImage: ( NSString * )imagepath;
- ( NSString * )previewedImage;
- ( void )displayPreview;
- ( void )addToCachedPreviews: ( NSDictionary *)cachedData;
- ( NSArray * )cachedPreviews;

- ( IBAction )editFile: ( id )sender;
- ( IBAction )openLocalFileInEditor: ( id )sender;
- ( IBAction )openRemoteFileInEditor: ( id )sender;
- ( void )editFile: ( NSString * )filepath remotePath: ( NSString * )remotepath withEditor: ( NSString * )editor;
- ( NSString * )remotePathForEditedDocumentWithLocalPath: (NSString *)docpath;
- ( NSDictionary* )editedDocumentWithRemotePath: ( NSString * )remotePath;

- ( NSArray * )editedDocuments;
- ( void )insertObject:(id)object inEditedDocumentsAtIndex: ( int )index;
- ( void )removeObjectFromEditedDocumentsAtIndex: ( int )index;

- ( void )addToEditedDocuments: ( NSString * )docpath remotePath: ( NSString * )rpath editor: ( NSString * )editor;
- ( void )removeEditedDocumentWithLocalPath: ( NSString * )docpath;

- ( IBAction )scp: ( id )sender;
- ( void )scpLocalItem: ( NSString * )item toHost: ( NSString * )rhost
              userName: ( NSString * )user;
- ( void )scpRemoteItem: ( NSString * )rdir fromHost: ( NSString * )rhost
            toLocalPath: ( NSString * )ldir userName: ( NSString * )user;
- ( void )scpFinished; /* SCPController delegate method */

@end
