//
//  SFTPLsCommand.m
//  Fugu
//
//  Created by Tim McIntosh on 2005.10.08.
//  Copyright 2005 __MyCompanyName__. All rights reserved.
//

#import "SFTPLsCommand.h"
#import "SFTPClientTaskController.h"
#import "CocoaDevKit/NSString+CocoaDevUsersAdditions.h"
#import "NSString+SFTPAdditions.h"

@implementation SFTPLsCommand

- (id)init { [self doesNotRecognizeSelector:@selector(init)]; return nil; }

- (SFTPLsCommand*)initWithPath:(NSString*)path
{
    self = [super init];
    if (self != nil)
    {
        fncolumn    = -1;
        datecolumn  = -1;
        ownercolumn =  2;
        mPath       = [path copy];
    }
    return self;
}

- (NSString*)command
{
    NSString* path = [[self controller] absolutePathFromString: mPath];
    NSString* command;

    double version = [[self controller] sshVersion];

    if (version < 3.5)
    {
        command = [NSString stringWithFormat: @"ls \"%@\"", path];
    }
    else if (version >= 3.9)
    {
        command = [NSString stringWithFormat: @"ls -la \"%@\"", path];
    }
    else
    {
        command = [NSString stringWithFormat: @"ls -l \"%@\"", path];
    }

    return command;
}

- (NSString*)displayName
{
    return NSLocalizedString(@"list files", @"ls command");
}

// XXX recode in objc, or find a different way to do this (by looking at version string, maybe? wouldn't work for server-side issues, though)
// should separate client issues (which could be determined from SSH version) from server issues.
#include <ctype.h>
#include <string.h>

- (BOOL)calculateColumns:(NSArray*)tokens
{
    unsigned int  tac   = [tokens count];
    NSMutableData*  argv  = [NSMutableData dataWithLength: sizeof (char*) * tac];
    const char**   targv = [argv mutableBytes];
    int            j;

    // Fill out targv - kluge to avoid rewriting the following C code (for now)
    for (j = 0; j < tac; j++)
    {
        targv[j] = [[tokens objectAtIndex: j] cString];
    }

    /*
     * find the filename column. The first line should contain
     * a filename '.' or './'.
     */
    for ( j = 0; j < tac; j++ ) {
        if ( strcmp( targv[ j ], "." ) == 0 || strcmp( targv[ j ], "./" ) == 0 ) {
            fncolumn = j;
            break;
        }
    }
    /* likewise, determine how many fields are used for the date.	*/
    /* based on code submitted by Hugues Martel.			*/
    if ( fncolumn == -1 ) {
        /* probably an invalid line, but might also be a VShell-like    */
        /* server at the root directory.				*/
        if ( isdigit( *targv[ 0 ] ) && strchr( targv[ 4 ], ':' ) != NULL ) {
            fncolumn = 5;
        } else if ( *targv[ 0 ] == 'd' || *targv[ 0 ] == '-'
                    && tac >= 9 ) {
            /* might also be OpenSSH on Cygwin, which doesn't display	*/
            /* a '.' or './' at the root directory. if so, handle it.	*/
            fncolumn = 8;
        }
    }
    if ( fncolumn == -1 || fncolumn > tac ) {
        return NO;			/* invalid output */
    }
    
    for ( j = ( fncolumn - 1 ); j >= 0; j-- ) {
        if ( isalpha( *targv[ j ] )) { 	/* we've found the column containing the month */
            datecolumn = j;
            if ( datecolumn != 5 ) {
                int             ind = 0;
                
                NSLog( @"datecolumn: %d\tdate: %s", datecolumn, targv[ j ] );
                for ( ind = 0; ind < tac; ind++ ) {
                    NSLog( @"targv[ %d ]: %s", ind, targv[ ind ] );
                }
                NSLog( @"line: %@", tokens );
            }
            break;
        }
    }
    if ( datecolumn < 0 ) {	/* potentially dealing with old OpenSSH version */
    if ( *targv[ 0 ] == '0' && strlen( targv[ 0 ] ) > 1
         && strchr( targv[ 4 ], ':' ) == NULL
         && fncolumn == 5 ) {
        datecolumn = ( fncolumn - 1 );
        ownercolumn = 1;
    }
    }
    if ( datecolumn >= tac || datecolumn < 0 ) {
        return NO;
    }

    return YES;  // success
}

// XXX needs cleanup.
- (NSMutableDictionary*)remoteObjectFromSFTPLine:(NSString*)line
{
    NSMutableArray*  tokens = [[line tokensSeparatedByCharactersFromSet: [NSCharacterSet whitespaceAndNewlineCharacterSet]] mutableCopy];
    NSMutableDictionary* infoDictionary = nil;

    if ([tokens count] < 1)  // XXX can this happen?
    {
        goto badline;
    }

    /* 
     * much of the abstraction in here to handle an arbitrary number of fields
     * was suggested by Hugues Martel. He used Obj-C calls. C calls are used here,
     * since otherwise there's a lot of unnecessary conversion.
     * Many thanks to Hugues for this contribution.
     */
    if (fncolumn < 0)
    {
#ifdef NOT_SUPPORTED
        // Need another way to determine this, because this can be fooled by a name containing '*' - TKM
        /* SSH.com's sftp gives true ls -lF output. 		*/
        /* Do we need to add other chars here (>, /, @, =)? 	*/
        if ( tac > 0 ) {
            if (( p = strchr( targv[ ( tac - 1 ) ], '*' )) != NULL 
                && *targv[ 0 ] == '-' ) {
                *p = '\0';
            }
        }
        
        /* SSH.com's sftp client writes dir name + : before listing */
        if ( tac == 1 && strcmp( targv[ 0 ], ".:" ) == 0 ) {
            fncolumn = 8;
            goto DOT_OR_DOTDOT;
        } else if ( tac == 1 ) {
            /* prevent crashes */
            goto DOT_OR_DOTDOT;
        }
#endif
        
        if (![self calculateColumns: tokens])
        {
            goto badline;// XXX
        }
    }
    
    if ([tokens count] < fncolumn)
    {
        goto badline; // XXX
    }

    NSString* name;

    // File name contains spaces  XXX - this won't work for names with multiple spaces
    // Could use regex to tokenize?
    if ([tokens count] > ( fncolumn + 1 ))
    {
        NSRange nameRange = NSMakeRange(fncolumn, [tokens count] - fncolumn);

        // Don't include symlink "-> ..." in name
        unsigned int linkIndex = [tokens indexOfObjectIdenticalTo: @"->" inRange: nameRange];

        if (linkIndex != NSNotFound)
        {
            nameRange.length = linkIndex - fncolumn;
        }

        name = [[tokens subarrayWithRange: nameRange] componentsJoinedByString: @" "];
    }
    else
    {
        name = [tokens objectAtIndex: fncolumn];
    }

    name = [name lastPathComponent];
    
    // Skip . & ..
    if (![name hasPrefix:@"."] ||
        ![name isEqualToString: @"."] &&
        ![name isEqualToString: @".."] &&
        ![name isEqualToString: @"./"] &&  // VShell output XXX - handle same as ls -F (which is TBD)
        ![name isEqualToString: @"../"])
    {
        NSString* type;
        NSString* perm;
        NSString* owner;
        NSString* group;
        NSString* date;

        date = [[tokens subarrayWithRange: NSMakeRange(datecolumn, fncolumn - datecolumn)] componentsJoinedByString: @" "];

        if (datecolumn == 1)
        {
            // dealing with a VShell server, probably.
            owner = group = @"N/A";

            // since VShell doesn't include a mode, we have to invent one
            // based on code submitted by Hugues Martel.

                                           /* directory */ /* file */
            perm = [name hasSuffix:@"/"] ? @"d---------" : @"----------";
        }
        else if (datecolumn > 2)
        {
            // probably some unix variant
            owner = [tokens objectAtIndex: ownercolumn];

            // possible to have group names containing spaces
            // NOTE: the "- 2" below accounts for the owner column & size column
            group = [[tokens subarrayWithRange: NSMakeRange(ownercolumn + 1, datecolumn - ownercolumn - 2)] componentsJoinedByString: @" "];
            
            perm = [tokens objectAtIndex: 0];

#ifdef NOT_SUPPORTED  // XXX - stringRepresentationOfOctalMode is not defined in project?
            /* handle old OpenSSH server by translating output */
            if (( datecolumn + 1 ) == fncolumn && [perm hasPrefix: @"0"])
            {
                perm = [perm stringRepresentationOfOctalMode];
            }
#endif
        }

        type = [perm sftpFileTypeStringFromPermissions];

        // Initialize file object

        infoDictionary = [NSMutableDictionary dictionary];

        [infoDictionary setObject: type  forKey: @"type"];
        [infoDictionary setObject: perm  forKey: @"perm"];
        [infoDictionary setObject: owner forKey: @"owner"];
        [infoDictionary setObject: group forKey: @"group"];
        if ( datecolumn >= 1 )
        {
            /* size always comes before date */
            [ infoDictionary setObject: [tokens objectAtIndex: datecolumn - 1]
                                forKey: @"size"];
        }
        [infoDictionary setObject: date  forKey: @"date"];
        [infoDictionary setObject: name  forKey: @"name"];
    }

    goto success;

badline:
    NSLog(@"Bad line: %@", tokens);
success:
    [tokens release];

    return infoDictionary;
}

- (void)parseResponse:(NSArray*)response
{
    //
    // Process response line-by-line
    //
    NSMutableArray* items = [NSMutableArray array];
    NSEnumerator*   lineEnumerator = [ response objectEnumerator ];
    NSString*       line;

    while ((line = [lineEnumerator nextObject]) != nil)
    {
        NSMutableDictionary* object;

        if ([line containsSftpError])
        {
            [[self controller] setError: line];
            goto error;  // XXX
        }

        if ((object = [self remoteObjectFromSFTPLine: line]) != nil )
        {
            [items addObject: object];
        }
    }

    [[self controller] setFiles: items];

error:
    [super parseResponse: response];
}

@end
