/*
 * Copyright (c) 2003 Regents of The University of Michigan.
 * All Rights Reserved.  See COPYRIGHT.
 */
 
#include <Security/SecBase.h>
#include <Security/SecKeychain.h>
#include <sys/types.h>
#include <sys/file.h>
#include <sys/param.h>
#include <errno.h>
#include <pwd.h>
#include <stdlib.h>
#include <string.h>
#include <syslog.h>
#include <unistd.h>
#include "keychain.h"

extern int			errno;

    char *
getpwdfromkeychain( const char *service, const char *account, OSStatus *error )
{
    OSStatus 			err;
    SecKeychainRef		skcref;
    UInt32 			len;
    char			*password;

    err = SecKeychainCopyDefault( &skcref );
    // XXX missing CFRelease on skcref?

    if ( err != noErr ) {
        syslog( LOG_ERR, "SecKeychainCopyDefault failed" );
        return( NULL );
    }

    void* buf;
    SecKeychainItemRef item;

    err = SecKeychainFindGenericPassword( skcref,
                strlen( service ), service,
                strlen( account ), account, &len, &buf, &item );

    // XXX - passing NULL for itemRef is causing crash with debugging libraries.

    password = (char*)buf;

    // XXX documentation says should use SecKeychainItemFreeContent on password.

    *error = err;
    switch ( err ) {
    case 0:
        break;
    case errSecItemNotFound:
        syslog( LOG_INFO, "keychain item not found" );
        return( NULL );
    case errSecAuthFailed:
        syslog( LOG_ERR, "authorization failed." );
        return( NULL );
    case errSecNoDefaultKeychain:
        syslog( LOG_INFO, "No default keychain!" );
        return( NULL );
    default:
        syslog( LOG_ERR, "unknown error" );
        return( NULL );
    }

    password[ len ] = '\0';
    
    /* returns malloc'd bytes which must be free'd */
    return( password );
}

    void
addpwdtokeychain( const char *service, const char *account, const char *password )
{
    OSStatus		err;
    SecKeychainRef	skcref;

    err = SecKeychainCopyDefault( &skcref );
    
    if ( err ) {
        syslog( LOG_ERR, "SecKeychainCopyDefault failed. Make sure you have a keychain available." );
        return;
    }
    
    err = SecKeychainAddGenericPassword( skcref,
                strlen( service ), service,
                strlen( account ), account,
                strlen( password ),
                ( const void * )password, NULL );
    
    switch ( err ) {
    case 0:
        break;
    case errSecDuplicateItem:
        syslog( LOG_INFO, "keychain item already exists." );
        break;
    case errSecAuthFailed:
        syslog( LOG_ERR, "authorization failed." );
        break;
    default:
        syslog( LOG_ERR, "unknown error adding password to keychain" );
        break;
    }
}